use super::constants::*;
use super::state::State;
use crate::gsa::Gsa;
use crate::*;
use glam::IVec2;

pub struct Stuff {
    pub gsa1: Gsa,
    pub gsa2: Gsa,
    pub mouse_pos: IVec2,
    pub tile_pos: IVec2,
    pub tile_pos2: IVec2,
    pub left_down: bool,
    pub middle_down: bool,
    pub right_down: bool,
    pub selected_tile: IVec2,
    pub current_layer: usize,
    pub all_layers: bool,
    pub state: State,
    pub input_buf: u16,
    pub current_map: u16,
    pub window_tile_size: IVec2,
}

impl Stuff {
    pub fn draw_input_window(&mut self, pos: IVec2, caption: &str) {
        self.gsa2.bg[2].draw_frame(pos, IVec2 { x: 7, y: 5 });
        self.gsa2.bg[2].tiles[pos.x as usize + 3][pos.y as usize + 1] = TILE_INPUT;
        self.gsa2.bg[2].tiles[pos.x as usize + 4][pos.y as usize + 1] = TILE_INPUT;
        self.gsa2.bg[2].tiles[pos.x as usize + 1][pos.y as usize + 3] = TILE_BUT_YES;
        self.gsa2.bg[2].tiles[pos.x as usize + 5][pos.y as usize + 3] = TILE_BUT_NO;
        self.gsa2.write_string(
            IVec2 {
                x: pos.x * 2 + 2,
                y: pos.y * 2 + 1,
            },
            caption,
        );
        self.gsa2.write_string(
            IVec2 {
                x: pos.x * 2 + 3,
                y: pos.y * 2 + 3,
            },
            "ID:0000",
        );
    }

    pub fn draw_toolbar(&mut self) {
        self.gsa2.bg[2].clear();
        self.gsa2.bg[3].clear();

        //vertical
        for y in 0..(self.window_tile_size.y - 1) {
            self.gsa2.bg[2].tiles[0][y as usize] = 0x7010;
        }
        self.gsa2.bg[2].tiles[0][BUT_SAVE] = 0x7211;
        self.gsa2.bg[2].tiles[0][BUT_CREATE] = TILE_BUT_CREATE;
        self.gsa2.bg[2].tiles[0][BUT_EXIT] = 0x7212;

        //layer buttons
        self.gsa2.bg[2].tiles[0][BUT_LAYER1] = if self.current_layer == 0 {
            0x7111
        } else {
            0x7011
        };
        self.gsa2.bg[2].tiles[0][BUT_LAYER2] = if self.current_layer == 1 {
            0x7112
        } else {
            0x7012
        };
        self.gsa2.bg[2].tiles[0][BUT_LAYER3] = if self.current_layer == 2 {
            0x7113
        } else {
            0x7013
        };
        self.gsa2.bg[2].tiles[0][BUT_LAYERS] = if self.all_layers { 0x7114 } else { 0x7014 };

	self.gsa2.bg[2].tiles[0][BUT_SHIFT_LEFT] = 0x7314;
	self.gsa2.bg[2].tiles[0][BUT_SHIFT_RIGHT] = 0x7315;
	self.gsa2.bg[2].tiles[0][BUT_SHIFT_UP] = 0x7316;
	self.gsa2.bg[2].tiles[0][BUT_SHIFT_DOWN] = 0x7317;

        self.gsa2.bg[2].size = IVec2 { x: 120, y: 68 }; //enough to cover 4k monitors? <_<
        self.gsa2.bg[3].size = IVec2 { x: 240, y: 136 };

        //horizontal
        let bottom = self.window_tile_size.y as usize - 1;
        self.gsa2.bg[2].tiles[0][bottom] = 0x7410;
        for x in 1..self.window_tile_size.x {
            self.gsa2.bg[2].tiles[x as usize][bottom] = 0x7310;
        }
	self.update_status_bar();
    }

    pub fn set_state(&mut self, state: State) {
        self.state = state;
        match state {
            State::Edit => {
                //update editing layers
                if self.all_layers {
                    self.gsa1.bg[0].active = true;
                    self.gsa1.bg[1].active = true;
                    self.gsa1.bg[2].active = true;
                } else {
                    self.gsa1.bg[0].active = self.current_layer == 0;
                    self.gsa1.bg[1].active = self.current_layer == 1;
                    self.gsa1.bg[2].active = self.current_layer == 2;
                }
                self.draw_toolbar();

                self.gsa2.bg[0].active = false;
                self.gsa2.bg[1].active = false;
                self.gsa1.sprite[SPR_CURSOR].priority = 3;
                self.gsa1.sprite[SPR_CURSOR].tile = TILE_CURSOR;
                self.gsa2.sprite[SPR_CURSOR].tile = EMPTY_TILE;
            }
            State::SelectTile => {
                self.draw_toolbar();
                self.gsa1.bg[0].active = false;
                self.gsa1.bg[1].active = false;
                self.gsa1.bg[2].active = false;
                self.gsa2.bg[0].active = true;
                self.gsa2.bg[1].active = true;
                self.gsa1.sprite[SPR_CURSOR].tile = EMPTY_TILE;
                self.gsa2.sprite[SPR_CURSOR].tile = TILE_CURSOR;
            }
            State::NewMapDialog => {
                self.gsa1.bg[0].active = false;
                self.gsa1.bg[1].active = false;
                self.gsa1.bg[2].active = false;
                self.gsa2.bg[0].active = false;
                self.gsa2.bg[1].active = false;
                self.gsa1.sprite[SPR_CURSOR].tile = EMPTY_TILE;
                self.gsa2.sprite[SPR_CURSOR].tile = EMPTY_TILE;
                self.draw_input_window(IVec2 { x: 15, y: 9 }, "Create Map");
            }
        }
    }

    pub fn update_input_window(&mut self, pos: IVec2, val: u16) {
        self.gsa2.write_string(
            IVec2 {
                x: pos.x * 2 + 6,
                y: pos.y * 2 + 3,
            },
            &format!("{:04x}", val),
        );
    }

    pub fn update_status_bar(&mut self) {
        let bottom = self.window_tile_size.y as usize - 1;
        let text_bottom = (bottom * 2 + 1) as i32;

        self.gsa2.write_string(
            IVec2 {
                x: 0,
                y: text_bottom,
            },
            &" ".repeat(self.window_tile_size.x as usize * 2),
        );
        self.gsa2.write_string(
            IVec2 {
                x: 1,
                y: text_bottom,
            },
            &format!("Map:{:04X}", self.current_map),
        );

        self.gsa2.write_string(
            IVec2 {
                x: 17,
                y: text_bottom,
            },
            &format!("Size:{}*{}", self.gsa1.bg[0].size.x, self.gsa1.bg[0].size.y),
        );

        self.gsa2.write_string(
            IVec2 {
                x: 33,
                y: text_bottom,
            },
            &format!("Cursor:{}*{}", self.tile_pos.x, self.tile_pos.y),
        );
    }
}
